@@ -2,8 +2,10 @@ import logging
 import re
 import string
 import threading
+import warnings
 from concurrent.futures import ThreadPoolExecutor, wait
 from enum import Enum, auto
+from functools import cached_property
 from typing import Any, Dict, List, Literal, Optional, Tuple, Type
 
 from google.api_core.exceptions import (
@@ -15,9 +17,14 @@ from google.api_core.exceptions import (
     ServiceUnavailable,
 )
 from google.cloud.aiplatform import telemetry
+from langchain_core._api.deprecation import deprecated
 from langchain_core.embeddings import Embeddings
 from langchain_core.language_models.llms import create_base_retry_decorator
-from langchain_core.pydantic_v1 import root_validator
+from pydantic import ConfigDict, model_validator
+from typing_extensions import Self
+from vertexai.generative_models._generative_models import (  # type: ignore[import-untyped]
+    SafetySettingsType as SafetySettingsType,
+)
 from vertexai.language_models import (  # type: ignore
     TextEmbeddingInput,
     TextEmbeddingModel,
@@ -39,6 +46,18 @@ _MAX_BATCH_SIZE = 250
 _MIN_BATCH_SIZE = 5
 
 
+EmbeddingTaskTypes = Literal[
+    "RETRIEVAL_QUERY",
+    "RETRIEVAL_DOCUMENT",
+    "SEMANTIC_SIMILARITY",
+    "CLASSIFICATION",
+    "CLUSTERING",
+    "QUESTION_ANSWERING",
+    "FACT_VERIFICATION",
+    "CODE_RETRIEVAL_QUERY",
+]
+
+
 class GoogleEmbeddingModelType(str, Enum):
     TEXT = auto()
     MULTIMODAL = auto()
@@ -57,6 +76,7 @@ class GoogleEmbeddingModelVersion(str, Enum):
     EMBEDDINGS_NOV_2023 = auto()
     EMBEDDINGS_DEC_2023 = auto()
     EMBEDDINGS_MAY_2024 = auto()
+    EMBEDDINGS_NOV_2024 = auto()
 
     @classmethod
     def _missing_(cls, value: Any) -> "GoogleEmbeddingModelVersion":
@@ -76,6 +96,8 @@ class GoogleEmbeddingModelVersion(str, Enum):
             or "text-multilingual-embedding-preview-0409" in value.lower()
         ):
             return GoogleEmbeddingModelVersion.EMBEDDINGS_MAY_2024
+        if "text-embedding-005" in value.lower():
+            return GoogleEmbeddingModelVersion.EMBEDDINGS_NOV_2024
 
         return GoogleEmbeddingModelVersion.EMBEDDINGS_JUNE_2023
 
@@ -100,24 +122,33 @@ class VertexAIEmbeddings(_VertexAICommon, Embeddings):
     # Instance context
     instance: Dict[str, Any] = {}  #: :meta private:
 
-    @root_validator()
-    def validate_environment(cls, values: Dict) -> Dict:
+    model_config = ConfigDict(
+        extra="forbid",
+        protected_namespaces=(),
+    )
+
+    @model_validator(mode="after")
+    def validate_environment(self) -> Self:
         """Validates that the python package exists in environment."""
-        cls._init_vertexai(values)
-        _, user_agent = get_user_agent(f"{cls.__name__}_{values['model_name']}")  # type: ignore
+        values = {
+            "project": self.project,
+            "location": self.location,
+            "credentials": self.credentials,
+            "api_transport": self.api_transport,
+            "api_endpoint": self.api_endpoint,
+            "default_metadata": self.default_metadata,
+        }
+        self._init_vertexai(values)
+        _, user_agent = get_user_agent(f"{self.__class__.__name__}_{self.model_name}")
         with telemetry.tool_context_manager(user_agent):
             if (
-                GoogleEmbeddingModelType(values["model_name"])
+                GoogleEmbeddingModelType(self.model_name)
                 == GoogleEmbeddingModelType.MULTIMODAL
             ):
-                values["client"] = MultiModalEmbeddingModel.from_pretrained(
-                    values["model_name"]
-                )
+                self.client = MultiModalEmbeddingModel.from_pretrained(self.model_name)
             else:
-                values["client"] = TextEmbeddingModel.from_pretrained(
-                    values["model_name"]
-                )
-        return values
+                self.client = TextEmbeddingModel.from_pretrained(self.model_name)
+        return self
 
     def __init__(
         self,
@@ -164,6 +195,10 @@ class VertexAIEmbeddings(_VertexAICommon, Embeddings):
             self.client.get_embeddings
         )
 
+    @cached_property
+    def _image_bytes_loader_client(self):
+        return ImageBytesLoader(project=self.project)
+
     @property
     def model_type(self) -> str:
         return GoogleEmbeddingModelType(self.model_name)
@@ -361,17 +396,7 @@ class VertexAIEmbeddings(_VertexAICommon, Embeddings):
         self,
         texts: List[str],
         batch_size: int = 0,
-        embeddings_task_type: Optional[
-            Literal[
-                "RETRIEVAL_QUERY",
-                "RETRIEVAL_DOCUMENT",
-                "SEMANTIC_SIMILARITY",
-                "CLASSIFICATION",
-                "CLUSTERING",
-                "QUESTION_ANSWERING",
-                "FACT_VERIFICATION",
-            ]
-        ] = None,
+        embeddings_task_type: Optional[EmbeddingTaskTypes] = None,
         dimensions: Optional[int] = None,
     ) -> List[List[float]]:
         """Embed a list of strings.
@@ -391,6 +416,8 @@ class VertexAIEmbeddings(_VertexAICommon, Embeddings):
                                           for Semantic Textual Similarity (STS).
                     CLASSIFICATION - Embeddings will be used for classification.
                     CLUSTERING - Embeddings will be used for clustering.
+                    CODE_RETRIEVAL_QUERY - Embeddings will be used for
+                                           code retrieval for Java and Python.
                     The following are only supported on preview models:
                     QUESTION_ANSWERING
                     FACT_VERIFICATION
@@ -432,7 +459,11 @@ class VertexAIEmbeddings(_VertexAICommon, Embeddings):
         return embeddings
 
     def embed_documents(
-        self, texts: List[str], batch_size: int = 0
+        self,
+        texts: List[str],
+        batch_size: int = 0,
+        *,
+        embeddings_task_type: EmbeddingTaskTypes = "RETRIEVAL_DOCUMENT",
     ) -> List[List[float]]:
         """Embed a list of documents.
 
@@ -445,9 +476,14 @@ class VertexAIEmbeddings(_VertexAICommon, Embeddings):
         Returns:
             List of embeddings, one for each text.
         """
-        return self.embed(texts, batch_size, "RETRIEVAL_DOCUMENT")
+        return self.embed(texts, batch_size, embeddings_task_type)
 
-    def embed_query(self, text: str) -> List[float]:
+    def embed_query(
+        self,
+        text: str,
+        *,
+        embeddings_task_type: EmbeddingTaskTypes = "RETRIEVAL_QUERY",
+    ) -> List[float]:
         """Embed a text.
 
         Args:
@@ -456,15 +492,54 @@ class VertexAIEmbeddings(_VertexAICommon, Embeddings):
         Returns:
             Embedding for the text.
         """
-        return self.embed([text], 1, "RETRIEVAL_QUERY")[0]
+        return self.embed([text], 1, embeddings_task_type)[0]
 
+    @deprecated(
+        since="2.0.1", removal="3.0.0", alternative="VertexAIEmbeddings.embed_images()"
+    )
     def embed_image(
-        self, image_path: str, contextual_text: Optional[str] = None
+        self,
+        image_path: str,
+        contextual_text: Optional[str] = None,
+        dimensions: Optional[int] = None,
     ) -> List[float]:
         """Embed an image.
 
         Args:
-            image_path: Path to image (local, Google Cloud Storage or web) to generate
+            image_path: Path to image (Google Cloud Storage or web) to generate
+            embeddings for.
+            contextual_text: Text to generate embeddings for.
+
+        Returns:
+            Embedding for the image.
+        """
+        warnings.warn(
+            "The `embed_image()` API will be deprecated and replaced by \
+            `embed_images()`. Change your usage to \
+            `embed_images([image_path1, image_path2])` and note\
+            that the result returned will be a list of image embeddings."
+        )
+        if self.model_type != GoogleEmbeddingModelType.MULTIMODAL:
+            raise NotImplementedError("Only supported for multimodal models")
+
+        image_loader = self._image_bytes_loader_client
+        bytes_image = image_loader.load_bytes(image_path)
+        image = Image(bytes_image)
+        result: MultiModalEmbeddingResponse = self.instance[
+            "get_embeddings_with_retry"
+        ](image=image, contextual_text=contextual_text, dimension=dimensions)
+        return result.image_embedding
+
+    def embed_images(
+        self,
+        uris: List[str],
+        contextual_text: Optional[str] = None,
+        dimensions: Optional[int] = None,
+    ) -> List[List[float]]:
+        """Embed a list of images.
+
+        Args:
+            uris: Paths to image (local, Google Cloud Storage or web) to generate
             embeddings for.
             contextual_text: Text to generate embeddings for.
 
@@ -474,10 +549,17 @@ class VertexAIEmbeddings(_VertexAICommon, Embeddings):
         if self.model_type != GoogleEmbeddingModelType.MULTIMODAL:
             raise NotImplementedError("Only supported for multimodal models")
 
-        image_loader = ImageBytesLoader()
-        bytes_image = image_loader.load_bytes(image_path)
-        image = Image(bytes_image)
-        result: MultiModalEmbeddingResponse = self.instance[
-            "get_embeddings_with_retry"
-        ](image=image, contextual_text=contextual_text)
-        return result.image_embedding
+        image_loader = self._image_bytes_loader_client
+
+        embeddings = []
+        for image_path in uris:
+            bytes_image = image_loader.load_bytes(image_path)
+            image = Image(bytes_image)
+            result: MultiModalEmbeddingResponse = self.instance[
+                "get_embeddings_with_retry"
+            ](image=image, contextual_text=contextual_text, dimension=dimensions)
+            embeddings.append(result.image_embedding)
+        return embeddings
+
+
+VertexAIEmbeddings.model_rebuild()
