@@ -32,6 +32,7 @@ class _BaseVertexAIVectorStore(VectorStore):
         embbedings: Optional[Embeddings] = None,
     ) -> None:
         """Constructor.
+
         Args:
             searcher: Object in charge of searching and storing the index.
             document_storage: Object in charge of storing and retrieving documents.
@@ -47,14 +48,15 @@ class _BaseVertexAIVectorStore(VectorStore):
         """Returns the embeddings object."""
         return self._embeddings
 
-    def similarity_search_with_score(
+    def similarity_search_with_score(  # type: ignore[override]
         self,
         query: str,
         k: int = 4,
         filter: Optional[List[Namespace]] = None,
         numeric_filter: Optional[List[NumericNamespace]] = None,
-    ) -> List[Tuple[Document, float]]:
+    ) -> List[Tuple[Document, Union[float, Dict[str, float]]]]:
         """Return docs most similar to query and their cosine distance from the query.
+
         Args:
             query: String query look up documents similar to.
             k: Number of Documents to return. Defaults to 4.
@@ -70,29 +72,41 @@ class _BaseVertexAIVectorStore(VectorStore):
                 the matching results. Please refer to
                 https://cloud.google.com/vertex-ai/docs/matching-engine/filtering#json
                 for more detail.
+
         Returns:
             List[Tuple[Document, float]]: List of documents most similar to
             the query text and cosine distance in float for each.
-            Lower score represents more similarity.
+            Higher score represents more similarity.
         """
 
-        embbedings = self._embeddings.embed_query(query)
+        embedding = self._embeddings.embed_query(query)
 
         return self.similarity_search_by_vector_with_score(
-            embedding=embbedings, k=k, filter=filter, numeric_filter=numeric_filter
+            embedding=embedding, k=k, filter=filter, numeric_filter=numeric_filter
         )
 
     def similarity_search_by_vector_with_score(
         self,
         embedding: List[float],
+        sparse_embedding: Optional[Dict[str, Union[List[int], List[float]]]] = None,
         k: int = 4,
+        rrf_ranking_alpha: float = 1,
         filter: Optional[List[Namespace]] = None,
         numeric_filter: Optional[List[NumericNamespace]] = None,
-    ) -> List[Tuple[Document, float]]:
+    ) -> List[Tuple[Document, Union[float, Dict[str, float]]]]:
         """Return docs most similar to the embedding and their cosine distance.
+
         Args:
             embedding: Embedding to look up documents similar to.
+            sparse_embedding: Sparse embedding dictionary which represents an embedding
+                as a list of dimensions and as a list of sparse values:
+                    ie. {"values": [0.7, 0.5], "dimensions": [10, 20]}
             k: Number of Documents to return. Defaults to 4.
+            rrf_ranking_alpha: Reciprocal Ranking Fusion weight, float between 0 and 1.0
+                Weights Dense Search VS Sparse Search, as an example:
+                - rrf_ranking_alpha=1: Only Dense
+                - rrf_ranking_alpha=0: Only Sparse
+                - rrf_ranking_alpha=0.7: 0.7 weighting for dense and 0.3 for sparse
             filter: Optional. A list of Namespaces for filtering
                 the matching results.
                 For example:
@@ -105,18 +119,44 @@ class _BaseVertexAIVectorStore(VectorStore):
                 the matching results. Please refer to
                 https://cloud.google.com/vertex-ai/docs/matching-engine/filtering#json
                 for more detail.
+
         Returns:
-            List[Tuple[Document, float]]: List of documents most similar to
-            the query text and cosine distance in float for each.
-            Lower score represents more similarity.
+            List[Tuple[Document, Union[float, Dict[str, float]]]]:
+            List of documents most similar to the query text and either
+            cosine distance in float for each or dictionary with both dense and sparse
+            scores if running hybrid search.
+            Higher score represents more similarity.
         """
+        if sparse_embedding is not None and not isinstance(sparse_embedding, dict):
+            raise ValueError(
+                "`sparse_embedding` should be a dictionary with the following format: "
+                "{'values': [0.7, 0.5, ...], 'dimensions': [10, 20, ...]}\n"
+                f"{type(sparse_embedding)} != {type({})}"
+            )
 
+        sparse_embeddings = [sparse_embedding] if sparse_embedding is not None else None
         neighbors_list = self._searcher.find_neighbors(
-            embeddings=[embedding], k=k, filter_=filter, numeric_filter=numeric_filter
+            embeddings=[embedding],
+            sparse_embeddings=sparse_embeddings,
+            k=k,
+            rrf_ranking_alpha=rrf_ranking_alpha,
+            filter_=filter,
+            numeric_filter=numeric_filter,
         )
+        if not neighbors_list:
+            return []
 
-        keys = [key for key, _ in neighbors_list[0]]
-        distances = [distance for _, distance in neighbors_list[0]]
+        keys = [elem["doc_id"] for elem in neighbors_list[0]]
+        if sparse_embedding is None:
+            distances = [elem["dense_score"] for elem in neighbors_list[0]]
+        else:
+            distances = [
+                {
+                    "dense_score": elem["dense_score"],
+                    "sparse_score": elem["sparse_score"],
+                }
+                for elem in neighbors_list[0]
+            ]
         documents = self._document_storage.mget(keys)
 
         if all(document is not None for document in documents):
@@ -129,6 +169,36 @@ class _BaseVertexAIVectorStore(VectorStore):
             message = f"Documents with ids: {missing_docs} not found in the storage"
             raise ValueError(message)
 
+    def delete(self, ids: Optional[List[str]] = None, **kwargs: Any) -> Optional[bool]:
+        """
+        Delete by vector ID.
+        Args:
+            ids (Optional[List[str]]): List of ids to delete.
+            **kwargs (Any): If added metadata={}, deletes the documents
+            that match the metadata filter and the parameter ids is not needed.
+        Returns:
+            Optional[bool]: True if deletion is successful.
+        Raises:
+            ValueError: If ids is None or an empty list.
+            RuntimeError: If an error occurs during the deletion process.
+        """
+        metadata = kwargs.get("metadata")
+        if (not ids and not metadata) or (ids and metadata):
+            raise ValueError(
+                "You should provide ids (as list of id's) or a metadata"
+                "filter for deleting documents."
+            )
+        if metadata:
+            ids = self._searcher.get_datapoints_by_filter(metadata=metadata)
+            if not ids:
+                return False
+        try:
+            self._searcher.remove_datapoints(datapoint_ids=ids)  # type: ignore[arg-type]
+            self._document_storage.mdelete(ids)  # type: ignore[arg-type]
+            return True
+        except Exception as e:
+            raise RuntimeError(f"Error during deletion: {str(e)}") from e
+
     def similarity_search(
         self,
         query: str,
@@ -138,6 +208,7 @@ class _BaseVertexAIVectorStore(VectorStore):
         **kwargs: Any,
     ) -> List[Document]:
         """Return docs most similar to query.
+
         Args:
             query: The string that will be used to search for similar documents.
             k: The amount of neighbors that will be retrieved.
@@ -152,6 +223,7 @@ class _BaseVertexAIVectorStore(VectorStore):
                 the matching results. Please refer to
                 https://cloud.google.com/vertex-ai/docs/matching-engine/filtering#json
                 for more detail.
+
         Returns:
             A list of k matching documents.
         """
@@ -166,22 +238,73 @@ class _BaseVertexAIVectorStore(VectorStore):
         self,
         texts: Iterable[str],
         metadatas: Union[List[dict], None] = None,
+        *,
+        ids: Optional[List[str]] = None,
         is_complete_overwrite: bool = False,
         **kwargs: Any,
     ) -> List[str]:
         """Run more texts through the embeddings and add to the vectorstore.
+
         Args:
             texts: Iterable of strings to add to the vectorstore.
             metadatas: Optional list of metadatas associated with the texts.
+            ids: Optional list of ids to be assigned to the texts in the index.
+                If None, unique ids will be generated.
+            is_complete_overwrite: Optional, determines whether this is an append or
+                overwrite operation. Only relevant for BATCH UPDATE indexes.
             kwargs: vectorstore specific parameters.
+
         Returns:
             List of ids from adding the texts into the vectorstore.
         """
 
-        # Makes sure is a list an can get the length, should we support iterables?
+        # Makes sure is a list and can get the length, should we support iterables?
         # metadata is a list so probably not?
         texts = list(texts)
-        ids = self._generate_unique_ids(len(texts))
+        embeddings = self._embeddings.embed_documents(texts)
+
+        return self.add_texts_with_embeddings(
+            texts=texts,
+            embeddings=embeddings,
+            metadatas=metadatas,
+            ids=ids,
+            is_complete_overwrite=is_complete_overwrite,
+            **kwargs,
+        )
+
+    def add_texts_with_embeddings(
+        self,
+        texts: List[str],
+        embeddings: List[List[float]],
+        metadatas: Union[List[dict], None] = None,
+        *,
+        sparse_embeddings: Optional[
+            List[Dict[str, Union[List[int], List[float]]]]
+        ] = None,
+        ids: Optional[List[str]] = None,
+        is_complete_overwrite: bool = False,
+        **kwargs: Any,
+    ) -> List[str]:
+        if ids is not None and len(set(ids)) != len(ids):
+            raise ValueError(
+                "All provided ids should be unique."
+                f"There are {len(ids)-len(set(ids))} duplicates."
+            )
+
+        if ids is not None and len(ids) != len(texts):
+            raise ValueError(
+                "The number of `ids` should match the number of `texts` "
+                f"{len(ids)} != {len(texts)}"
+            )
+
+        if isinstance(embeddings, list) and len(embeddings) != len(texts):
+            raise ValueError(
+                "The number of `embeddings` should match the number of `texts` "
+                f"{len(embeddings)} != {len(texts)}"
+            )
+
+        if ids is None:
+            ids = self._generate_unique_ids(len(texts))
 
         if metadatas is None:
             metadatas = [{}] * len(texts)
@@ -199,10 +322,13 @@ class _BaseVertexAIVectorStore(VectorStore):
 
         self._document_storage.mset(list(zip(ids, documents)))
 
-        embeddings = self._embeddings.embed_documents(texts)
-
         self._searcher.add_to_index(
-            ids, embeddings, metadatas, is_complete_overwrite, **kwargs
+            ids=ids,
+            embeddings=embeddings,
+            sparse_embeddings=sparse_embeddings,
+            metadatas=metadatas,
+            is_complete_overwrite=is_complete_overwrite,
+            **kwargs,
         )
 
         return ids
@@ -225,6 +351,7 @@ class _BaseVertexAIVectorStore(VectorStore):
     @classmethod
     def _get_default_embeddings(cls) -> Embeddings:
         """This function returns the default embedding.
+
         Returns:
             Default TensorflowHubEmbeddings to use.
         """
@@ -247,8 +374,10 @@ class _BaseVertexAIVectorStore(VectorStore):
 
     def _generate_unique_ids(self, number: int) -> List[str]:
         """Generates a list of unique ids of length `number`
+
         Args:
             number: Number of ids to generate.
+
         Returns:
             List of unique ids.
         """
@@ -275,6 +404,7 @@ class VectorSearchVectorStore(_BaseVertexAIVectorStore):
         **kwargs: Any,
     ) -> "VectorSearchVectorStore":
         """Takes the object creation out of the constructor.
+
         Args:
             project_id: The GCP project id.
             region: The default location making the API calls. It must have
@@ -293,8 +423,9 @@ class VectorSearchVectorStore(_BaseVertexAIVectorStore):
                 index must be compatible with stream/batch updates.
             kwargs: Additional keyword arguments to pass to
                 VertexAIVectorSearch.__init__().
+
         Returns:
-            A configured VertexAIVectorSearch with the texts added to the index.
+            A configured VertexAIVectorSearch.
         """
 
         sdk_manager = VectorSearchSDKManager(
@@ -342,6 +473,7 @@ class VectorSearchVectorStoreDatastore(_BaseVertexAIVectorStore):
         embedding: Optional[Embeddings] = None,
         stream_update: bool = False,
         datastore_client_kwargs: Optional[Dict[str, Any]] = None,
+        exclude_from_indexes: Optional[List[str]] = None,
         datastore_kind: str = "document_id",
         datastore_text_property_name: str = "text",
         datastore_metadata_property_name: str = "metadata",
@@ -349,7 +481,7 @@ class VectorSearchVectorStoreDatastore(_BaseVertexAIVectorStore):
     ) -> "VectorSearchVectorStoreDatastore":
         """Takes the object creation out of the constructor.
 
-        # Args:
+        Args:
             project_id: The GCP project id.
             region: The default location making the API calls. It must have
                 the same location as the GCS bucket and must be regional.
@@ -366,6 +498,10 @@ class VectorSearchVectorStoreDatastore(_BaseVertexAIVectorStore):
                 index must be compatible with stream/batch updates.
             kwargs: Additional keyword arguments to pass to
                 VertexAIVectorSearch.__init__().
+            exclude_from_indexes: Fields to exclude from datastore indexing
+
+        Returns:
+            A configured VectorSearchVectorStoreDatastore.
         """
 
         sdk_manager = VectorSearchSDKManager(
@@ -389,11 +525,14 @@ class VectorSearchVectorStoreDatastore(_BaseVertexAIVectorStore):
 
         datastore_client = sdk_manager.get_datastore_client(**datastore_client_kwargs)
 
+        if exclude_from_indexes is None:
+            exclude_from_indexes = []
         document_storage = DataStoreDocumentStorage(
             datastore_client=datastore_client,
             kind=datastore_kind,
             text_property_name=datastore_text_property_name,
             metadata_property_name=datastore_metadata_property_name,
+            exclude_from_indexes=exclude_from_indexes,
         )
 
         return cls(
