@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from functools import cached_property
 from typing import Any, Dict, List, Optional, Union
 
 from google.cloud.aiplatform import telemetry
@@ -9,12 +10,13 @@ from langchain_core.messages import AIMessage, BaseMessage
 from langchain_core.outputs import ChatResult, LLMResult
 from langchain_core.outputs.chat_generation import ChatGeneration
 from langchain_core.outputs.generation import Generation
-from langchain_core.pydantic_v1 import BaseModel, Field
-from vertexai.preview.vision_models import (  # type: ignore[import-untyped]
+from pydantic import BaseModel, ConfigDict, Field
+from vertexai.vision_models import (  # type: ignore[import-untyped]
     GeneratedImage,
+    Image,
     ImageGenerationModel,
+    ImageTextModel,
 )
-from vertexai.vision_models import Image, ImageTextModel  # type: ignore[import-untyped]
 
 from langchain_google_vertexai._image_utils import (
     ImageBytesLoader,
@@ -29,7 +31,7 @@ from langchain_google_vertexai._utils import get_user_agent
 class _BaseImageTextModel(BaseModel):
     """Base class for all integrations that use ImageTextModel"""
 
-    cached_client: Any = Field(default=None)
+    cached_client: Any = Field(default=None, exclude=True)
     model_name: str = Field(default="imagetext@001")
     """ Name of the model to use"""
     number_of_results: int = Field(default=1)
@@ -39,6 +41,8 @@ class _BaseImageTextModel(BaseModel):
     project: Union[str, None] = Field(default=None)
     """Google cloud project"""
 
+    model_config = ConfigDict(protected_namespaces=())
+
     @property
     def client(self) -> ImageTextModel:
         if self.cached_client is None:
@@ -47,6 +51,10 @@ class _BaseImageTextModel(BaseModel):
             )
         return self.cached_client
 
+    @cached_property
+    def _image_bytes_loader_client(self):
+        return ImageBytesLoader(project=self.project)
+
     def _get_image_from_message_part(self, message_part: str | Dict) -> Image | None:
         """Given a message part obtain a image if the part represents it.
 
@@ -60,7 +68,7 @@ class _BaseImageTextModel(BaseModel):
         image_str = get_image_str_from_content_part(message_part)
 
         if isinstance(image_str, str):
-            loader = ImageBytesLoader(project=self.project)
+            loader = self._image_bytes_loader_client
             image_bytes = loader.load_bytes(image_str)
             return Image(image_bytes=image_bytes)
         else:
@@ -95,7 +103,7 @@ class _BaseImageTextModel(BaseModel):
     def _prepare_params(self, **kwargs: Any) -> Dict[str, Any]:
         params = self._default_params
         for key, value in kwargs.items():
-            if key in params and value is not None:
+            if value is not None:
                 params[key] = value
         return params
 
@@ -108,6 +116,7 @@ class _BaseVertexAIImageCaptioning(_BaseImageTextModel):
         image: Image,
         number_of_results: Optional[int] = None,
         language: Optional[str] = None,
+        **kwargs,
     ) -> List[str]:
         """Uses the sdk methods to generate a list of captions.
 
@@ -121,7 +130,7 @@ class _BaseVertexAIImageCaptioning(_BaseImageTextModel):
         """
         with telemetry.tool_context_manager(self._user_agent):
             params = self._prepare_params(
-                number_of_results=number_of_results, language=language
+                number_of_results=number_of_results, language=language, **kwargs
             )
             captions = self.client.get_captions(image=image, **params)
             return captions
@@ -166,7 +175,7 @@ class VertexAIImageCaptioning(_BaseVertexAIImageCaptioning, BaseLLM):
         Returns:
             List of generations
         """
-        image_loader = ImageBytesLoader(project=self.project)
+        image_loader = self._image_bytes_loader_client
         image_bytes = image_loader.load_bytes(prompt)
         image = Image(image_bytes=image_bytes)
         caption_list = self._get_captions(image=image, **kwargs)
@@ -222,7 +231,7 @@ class VertexAIImageCaptioningChat(_BaseVertexAIImageCaptioning, BaseChatModel):
                 "{'type': 'image_url', 'image_url': {'image': <image_str>}}"
             )
 
-        captions = self._get_captions(image, **messages[0].additional_kwargs)
+        captions = self._get_captions(image, **messages[0].additional_kwargs, **kwargs)
 
         generations = [
             ChatGeneration(message=AIMessage(content=caption)) for caption in captions
@@ -285,7 +294,7 @@ class VertexAIVisualQnAChat(_BaseImageTextModel, BaseChatModel):
             )
 
         answers = self._ask_questions(
-            image=image, query=user_question, **messages[0].additional_kwargs
+            image=image, query=user_question, **messages[0].additional_kwargs, **kwargs
         )
 
         generations = [
@@ -315,7 +324,7 @@ class VertexAIVisualQnAChat(_BaseImageTextModel, BaseChatModel):
 class _BaseVertexAIImageGenerator(BaseModel):
     """Base class form generation and edition of images."""
 
-    cached_client: Any = Field(default=None)
+    cached_client: Any = Field(default=None, exclude=True)
     model_name: str = Field(default="imagegeneration@002")
     """Name of the base model"""
     negative_prompt: Union[str, None] = Field(default=None)
@@ -334,6 +343,8 @@ class _BaseVertexAIImageGenerator(BaseModel):
     project: Union[str, None] = Field(default=None)
     """Google cloud project id"""
 
+    model_config = ConfigDict(protected_namespaces=())
+
     @property
     def client(self) -> ImageGenerationModel:
         if not self.cached_client:
@@ -352,12 +363,16 @@ class _BaseVertexAIImageGenerator(BaseModel):
             "seed": self.seed,
         }
 
+    @cached_property
+    def _image_bytes_loader_client(self):
+        return ImageBytesLoader(project=self.project)
+
     def _prepare_params(self, **kwargs: Any) -> Dict[str, Any]:
         params = self._default_params
         mapping = {"number_of_results": "number_of_images"}
         for key, value in kwargs.items():
             key = mapping.get(key, key)
-            if key in params and value is not None:
+            if value is not None:
                 params[key] = value
         return {k: v for k, v in params.items() if v is not None}
 
@@ -392,7 +407,7 @@ class _BaseVertexAIImageGenerator(BaseModel):
             List of b64 encoded strings.
         """
         with telemetry.tool_context_manager(self._user_agent):
-            image_loader = ImageBytesLoader(project=self.project)
+            image_loader = self._image_bytes_loader_client
             image_bytes = image_loader.load_bytes(image_str)
             image = Image(image_bytes=image_bytes)
             generation_result = self.client.edit_image(
@@ -459,9 +474,13 @@ class VertexAIImageGeneratorChat(_BaseVertexAIImageGenerator, BaseChatModel):
 
         # Only one message allowed with one text part.
         user_query = None
-        is_valid = len(messages) == 1 and len(messages[0].content) == 1
-        if is_valid:
-            user_query = get_text_str_from_content_part(messages[0].content[0])
+
+        if len(messages) == 1:
+            if isinstance(messages[0].content, str):
+                user_query = messages[0].content
+            elif len(messages[0].content) == 1:
+                user_query = get_text_str_from_content_part(messages[0].content[0])
+
         if user_query is None:
             raise ValueError(
                 "Only one message with one text part allowed for image generation"
@@ -469,7 +488,7 @@ class VertexAIImageGeneratorChat(_BaseVertexAIImageGenerator, BaseChatModel):
             )
 
         image_str_list = self._generate_images(
-            prompt=user_query, **messages[0].additional_kwargs
+            prompt=user_query, **messages[0].additional_kwargs, **kwargs
         )
         image_content_part_list = [
             create_image_content_part(image_str=image_str)
@@ -518,7 +537,10 @@ class VertexAIImageEditorChat(_BaseVertexAIImageGenerator, BaseChatModel):
             )
 
         image_str_list = self._edit_images(
-            image_str=image_str, prompt=user_query, **messages[0].additional_kwargs
+            image_str=image_str,
+            prompt=user_query,
+            **messages[0].additional_kwargs,
+            **kwargs,
         )
         image_content_part_list = [
             create_image_content_part(image_str=image_str)
